﻿using System;
using System.Threading;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using AForge.Video;
using AForge.Video.DirectShow;
using AForge.Video.FFMPEG;
using AForge.Video.VFW;

namespace InterfaceWebCam
{

    public partial class Form1 : Form
    {
        private FilterInfoCollection VideoCaptureDevices;
        public int time;
        public bool snapshotTrigger = false;

        private VideoCaptureDevice FinalVideo = null;
        private VideoCaptureDeviceForm captureDevice;
        private Bitmap video;
        //private AVIWriter AVIwriter = new AVIWriter();
        private VideoFileWriter FileWriter = new VideoFileWriter();
        private SaveFileDialog saveAvi;

        public Form1()
        {
            InitializeComponent();
        }

        private void btnConnect_Click(object sender, EventArgs e)
        {
            if (captureDevice.ShowDialog(this) == DialogResult.OK)
            {
                FinalVideo = captureDevice.VideoDevice;
                FinalVideo.NewFrame += new NewFrameEventHandler(FinalVideo_NewFrame);
                FinalVideo.Start();

                btnStart.Enabled = true;
                btnStartRecording.Enabled = true;
                btnStopRecording.Enabled = false;
                btnConnect.Enabled = false;
                btnStop.Enabled = false;
            }
        }

        private void btnStart_Click(object sender, EventArgs e)
        {
            //This code makes sure an integer value was typed into timeBox, otherwise it returns an error
            int parsedValue;

            if (!int.TryParse(timeBox.Text, out parsedValue))
            {
                MessageBox.Show("Interval Error. Must be an integer.");
                return;
            }

            //Assuming the above check worked out, we disable some buttons and the textboxes
            pictureBox1.Visible = false;
            pictureBox2.Visible = true;
            btnStop.Enabled = true;
            btnStart.Enabled = false;
            btnStartRecording.Enabled = false;
            timer1.Enabled = true;
            timer2.Enabled = true;
            snapshotTrigger = true;
        }

        private void btnStop_Click(object sender, EventArgs e)
        {
            StopSnapshot();
        }

        void StopSnapshot()
        {
            btnStart.Enabled = true;
            btnStop.Enabled = false;
            if (btnConnect.Enabled == false) btnStartRecording.Enabled = true;
            pictureBox1.Visible = true;
            pictureBox2.Visible = false;
            timer1.Enabled = false;
            timer2.Enabled = false;
            snapshotTrigger = false;
        }

        private void btnRecording_Click(object sender, EventArgs e)
        {
            saveAvi = new SaveFileDialog();
            saveAvi.Filter = "Avi Files (*.avi)|*.avi";
            if (saveAvi.ShowDialog() == System.Windows.Forms.DialogResult.OK)
            {
                int h = captureDevice.VideoDevice.VideoResolution.FrameSize.Height;
                int w = captureDevice.VideoDevice.VideoResolution.FrameSize.Width;
                FileWriter.Open(saveAvi.FileName, w, h, 25, VideoCodec.Default, 5000000);
                FileWriter.WriteVideoFrame(video);

                //AVIwriter.Open(saveAvi.FileName, w, h);
                btnStartRecording.Enabled = false;
                btnStopRecording.Enabled = true;
                recordLabel.Visible = true;
                pictureBox1.Visible = true;
                pictureBox2.Visible = false;
                btnStart.Enabled = false;
            }
        }

        private void btnStopRecording_Click(object sender, EventArgs e)
        {
            if (btnStopRecording.Enabled == true)
            {
                btnStopRecording.Enabled = false;
                btnStart.Enabled = true;
                recordLabel.Visible = false;
                if (FinalVideo == null)
                { return; }
                if (FinalVideo.IsRunning)
                {
                    //this.FinalVideo.Stop();
                    FileWriter.Close();
                    //this.AVIwriter.Close();
                    pictureBox1.Image = null;
                    //pictureBox2.Image = null;
                    btnStartRecording.Enabled = true;
                }
            }
            else
            {
                this.FinalVideo.Stop();
                FileWriter.Close();
                //this.AVIwriter.Close();
                pictureBox1.Image = null;
            }
        }

        private void btnInterval_Click(object sender, EventArgs e)
        {
            //This code makes sure an integer value was typed into newTimeBox, otherwise it returns an error
            int parsedValue;

            if (!int.TryParse(newTimeBox.Text, out parsedValue))
            {
                MessageBox.Show("Interval value must be an integer.");
                return;
            }
            else
            {
                timeBox.Text = newTimeBox.Text;
                time = Convert.ToInt32(timeBox.Text);
                timer2.Interval = time;
            }
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            VideoCaptureDevices = new FilterInfoCollection(FilterCategory.VideoInputDevice);
            captureDevice = new VideoCaptureDeviceForm();
        }

        private void Form1_FormClosing(object sender, FormClosingEventArgs e)
        {
            timer1.Enabled = false;
            timer2.Enabled = false;

            if (FinalVideo == null)
            { return; }
            if (FinalVideo.IsRunning)
            {
                FileWriter.Close();
                MessageBox.Show("Exiting program cleanly."); //See comment on next line! This is important!!
                //Thread.Sleep(5000); //This doesn't work. But if the MessageBox above is removed, the program returns an error when it closes and a process remains running! 
                this.FinalVideo.Stop();
            }
        }

        private void timer1_Tick(object sender, EventArgs e)
        {
            if (snapshotTrigger == true)
                snapshotTrigger = false;
        }

        private void timer2_Tick(object sender, EventArgs e)
        {
           if (snapshotTrigger == false)
                snapshotTrigger = true;
        }

        void FinalVideo_NewFrame(object sender, NewFrameEventArgs eventArgs)
        {
            this.Invoke((MethodInvoker)delegate
            {
                try
                {
                    if (video != null)
                    {
                        video.Dispose();
                    }
                    video = new Bitmap(eventArgs.Frame);
                }
                catch { }

                this.Invoke((MethodInvoker)delegate
                {
                    if (btnStopRecording.Enabled == true)
                    {
                        video = (Bitmap)eventArgs.Frame.Clone();
                        pictureBox1.Image = (Bitmap)eventArgs.Frame.Clone();
                        //AVIwriter.Quality = 0;
                        FileWriter.WriteVideoFrame(video);
                        //AVIwriter.AddFrame(video);
                    }
                    else
                    {
                        video = (Bitmap)eventArgs.Frame.Clone();
                        pictureBox1.Image = (Bitmap)eventArgs.Frame.Clone();
                        if (snapshotTrigger == true)
                        {
                            pictureBox2.Image = null;
                            pictureBox2.Image = (Bitmap)eventArgs.Frame.Clone();
                        }
                    }
                });
                GC.Collect();
            });
        }
    }
}




/*
        void FinalVideo_NewFrame(object sender, NewFrameEventArgs eventArgs)
        {
            if (butStop.Enabled == true)
            {
                video = (Bitmap)eventArgs.Frame.Clone();
                pictureBox1.Image = (Bitmap)eventArgs.Frame.Clone();
                //if (snapshotTrigger == false) pictureBox2.Image = (Bitmap)eventArgs.Frame.Clone();
                //AVIwriter.Quality = 0;
                FileWriter.WriteVideoFrame(video);
                //AVIwriter.AddFrame(video);
            }
            else
            {
                video = (Bitmap)eventArgs.Frame.Clone();
                pictureBox1.Image = (Bitmap)eventArgs.Frame.Clone();
                if (snapshotTrigger == true)
                {
                    pictureBox2.Image = null;
                    pictureBox2.Image = (Bitmap)eventArgs.Frame.Clone();
                }
            }
        }
*/ 
